-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: DPS")

L:RegisterTranslations("enUS", function() return {
	["Damage per Second"] = true,
	["DPS"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Damage per Second"] = "초당 피해량",
	["DPS"] = "DPS",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local playerName = nil
local data = nil
local broadcastdirty = nil
local dirty = nil
local lastUpdateTime = {}
local seconds = nil
local damageDone = nil
local moduleName = "DPS"
local isInCombat = {}

local roster = AceLibrary("Roster-2.1")
local parser = ParserLib:GetInstance("1.1")

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 35510 $"):sub(12, -3)) or 1
mod.version = 4

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleRegister()
	playerName = UnitName("player")
	data = self:new()
end

function mod:OnModuleEnable()
	parser:RegisterInfoType( "Violation "..moduleName, "hit", function( infoType, info ) self:OnParserEvent( infoType, info ) end )
	parser:RegisterInfoType( "Violation "..moduleName, "drain",function( infoType, info ) self:OnParserEvent( infoType, info ) end  )
	parser:RegisterInfoType( "Violation "..moduleName, "leech", function( infoType, info ) self:OnParserEvent( infoType, info ) end  )
	self:RegisterEvent("PLAYER_REGEN_ENABLED") -- handle player dps time via regen enabled
	self:ScheduleRepeatingEvent( self.CheckPartyCombat, 1 )
end

function mod:OnModuleDisable()
	parser:UnregisterAllInfoTypes( "Violation "..moduleName )
end

function mod:OnReset()
	if type(data) == "table" then
		data = self:del(data)
	end

	data = self:new()
	data[playerName] = 0
	
	if type(seconds) == "table" then
		seconds = self:del(seconds)
	end
	seconds = self:new()
	
	if type(damageDone) == "table" then
		damageDone = self:del(damageDone)
	end
	damageDone = self:new()
	dirty = nil
	broadcastdirty = nil
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetRevision() return self.revision end
function mod:GetDisplayName() return L["Damage per Second"] end
function mod:GetShortDisplayName() return L["DPS"] end
function mod:GetBroadcastData()
	broadcastdirty = nil
	return data[playerName]
end

function mod:IsDirty()
	return dirty
end

function mod:ResetDirty()
	dirty = nil
end

function mod:IsBroadcastDirty()
	return broadcastdirty
end

function mod:OnRemoteData(sender, input)
	data[sender] = input
	dirty = true
end

function mod:PLAYER_REGEN_ENABLED()
	lastUpdateTime[playerName] = nil
end

function mod:OnParserEvent( infoType, info )
	local name, amount
	if infoType == "hit" or ( ( infoType == "drain" or infoType == "leech" ) and info.attribute == "Health" ) then
		amount = info.amount
		name = info.source
	end
	if not name or name == ParserLib_NONE or not amount or amount == 0 then
		return
	end

	local unitid
	if name == ParserLib_SELF then
		unitid = "player"
	else
		unitid = roster:GetUnitIDFromName( name )
	end
	if not unitid then return end	
	
	local newTime = GetTime()
	local timeSinceLastUpdate

	if not lastUpdateTime[name] then
		timeSinceLastUpdate = 0.1
	else
		timeSinceLastUpdate = newTime - lastUpdateTime[name]
	end
	lastUpdateTime[name] = newTime
	seconds[name] = (seconds[name] or 0 ) + timeSinceLastUpdate

	damageDone[name] = ( damageDone[name] or 0 ) + amount		
		
	local newDPS = math.floor(damageDone[name] / seconds[name])

	if name == ParserLib_SELF then
		broadcastdirty = newDPS - (data[playerName] or 0)
		data[playerName] = newDPS
		dirty = true
	elseif not Violation:UserViolation( name, moduleName ) then
		self:OnRemoteData( name, newDPS )			
	end
end

function mod:GetData(player)
	if player then
		return data[player]
	else
		return data
	end
end

function mod:CheckPartyCombat()
	for name, unit in pairs( roster.roster ) do
		if isInCombat[name] and not UnitAffectingCombat( unit.unitid ) then
			isInCombat[name] = nil
			lastUpdateTime[name] = nil
		elseif not isInCombat[name] and UnitAffectingCombat( unit.unitid ) then
			isInCombat[name] = true
		end
	end
end
