-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: Damage")

L:RegisterTranslations("enUS", function() return {
	["Damage done"] = true,
	["D"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Damage done"] = "입힌 피해량",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local playerName = nil
local data = nil
local broadcastdirty = 0
local dirty = nil
local moduleName = "Damage"

local parser = ParserLib:GetInstance("1.1")
local roster = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 35510 $"):sub(12, -3)) or 1
mod.version = 4

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleRegister()
	playerName = UnitName("player")
	data = self:new()
end

function mod:OnModuleEnable()
	parser:RegisterInfoType( "Violation "..moduleName, "hit", function( infoType, info ) self:OnParserEvent( infoType, info ) end )
	parser:RegisterInfoType( "Violation "..moduleName, "drain",function( infoType, info ) self:OnParserEvent( infoType, info ) end  )
	parser:RegisterInfoType( "Violation "..moduleName, "leech", function( infoType, info ) self:OnParserEvent( infoType, info ) end  )
end

function mod:OnModuleDisable()
	parser:UnregisterAllInfoTypes( "Violation "..moduleName )
end

function mod:OnReset()
	if type(data) == "table" then
		data = self:del(data)
	end

	data = self:new()
	data[playerName] = 0
	dirty = nil
	broadcastdirty = 0
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetRevision() return self.revision end
function mod:GetDisplayName() return L["Damage done"] end
function mod:GetShortDisplayName() return L["D"] end
function mod:GetBroadcastData()
	local x = broadcastdirty
	broadcastdirty = 0
	return x
end

function mod:IsBroadcastDirty()
	return broadcastdirty > 0 and true or false
end

function mod:IsDirty()
	return dirty
end

function mod:ResetDirty()
	dirty = nil
end

function mod:OnRemoteData(sender, input)
	data[sender] = (data[sender] or 0) + input
	dirty = true
end

function mod:OnParserEvent( infoType, info )
	local source, amount
	if infoType == "hit" or ( ( infoType == "drain" or infoType == "leech" ) and info.attribute == "Health" ) then
		amount = info.amount
		source = info.source
	end
	if not source or source == ParserLib_NONE or not amount or amount == 0 then
		return
	end
	if source == ParserLib_SELF then
		data[playerName] = data[playerName] + amount
		broadcastdirty = broadcastdirty + amount
		dirty = true
	else
		local unitid = roster:GetUnitIDFromName( source )
		if not unitid then return end
		if not Violation:UserViolation( source, moduleName ) then
			self:OnRemoteData( source, amount )			
		end
	end
end

function mod:GetData(player)
	if player then
		return data[player]
	else
		return data
	end
end

