-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: HPS")

L:RegisterTranslations("enUS", function() return {
	["Healing per Second"] = true,
	["HPS"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Healing per Second"] = "초당 치유량",
	["HPS"] = "HPS",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local playerName = nil

local data = nil
local dirty = nil
local broadcastdirty = nil
local lastUpdateTime = {}
local seconds = nil
local healingDone = nil
local moduleName = "HPS"
local isInCombat = {}


local roster = AceLibrary("Roster-2.1")
local parser = ParserLib:GetInstance("1.1")

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 35576 $"):sub(12, -3)) or 1
mod.version = 4

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleRegister()
	playerName = UnitName("player")
	data = self:new()
end

function mod:OnModuleEnable()
	parser:RegisterInfoType( "Violation "..moduleName, "heal", function( infoType, info ) self:OnParserEvent( infoType, info ) end )
	parser:RegisterInfoType( "Violation "..moduleName, "drain",function( infoType, info ) self:OnParserEvent( infoType, info ) end  )
	parser:RegisterInfoType( "Violation "..moduleName, "leech", function( infoType, info ) self:OnParserEvent( infoType, info ) end  )
	
	self:RegisterEvent("PLAYER_REGEN_ENABLED") -- handle player dps time via regen enabled
	self:ScheduleRepeatingEvent( self.CheckPartyCombat, 1 )
end

function mod:OnModuleDisable()
	parser:UnregisterAllInfoTypes( "Violation "..moduleName )
end

function mod:OnReset()
	if type(data) == "table" then
		data = self:del(data)
	end

	data = self:new()
	data[playerName] = 0
	
	if type(seconds) == "table" then
		seconds = self:del(seconds)
	end
	seconds = self:new()
	
	if type(healingDone) == "table" then
		healingDone = self:del(healingDone)
	end
	healingDone = self:new()

	dirty = nil
	broadcastdirty = nil
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetRevision() return self.revision end
function mod:GetDisplayName() return L["Healing per Second"] end
function mod:GetShortDisplayName() return L["HPS"] end
function mod:GetBroadcastData()
	broadcastdirty = nil
	return data[playerName]
end

function mod:IsBroadcastDirty()
	return broadcastdirty
end

function mod:IsDirty()
	return dirty
end

function mod:ResetDirty()
	dirty = nil
end

function mod:OnRemoteData(sender, input)
	data[sender] = input
	dirty = true
end

function mod:PLAYER_REGEN_ENABLED()
	lastUpdateTime[playerName] = nil
end

local function getOverheal( unit, amount )
	local overheal = 0
	if unit and UnitExists(unit) then
		local d = UnitHealthMax(unit)-UnitHealth(unit)
		if d < amount then
			overheal = amount - d
		end
	end
	return overheal
end

function mod:OnParserEvent( infoType, info )
	local name, amount, victim
	if infoType == "heal" or ( ( infoType == "drain" or infoType == "leech" ) and info.attribute == "Health" ) then
		amount = info.amount
		name = info.source
		victim = info.victim
	end
	if not name or name == ParserLib_NONE or not victim or victim == ParserLib_NONE or not amount or amount == 0 then
		return
	end
	
	local unitid
	if name == ParserLib_SELF then
		unitid = "player"
	else
		unitid = roster:GetUnitIDFromName( name )
	end
	if not unitid then -- we're only interested in info from known friendlies
		return
	end

	local victimid
	if victimid == ParserLib_SELF then
		victimid = "player"
	else
		victimid = roster:GetUnitIDFromName( victim ) 
	end
	
	local newTime = GetTime()
	local timeSinceLastUpdate

	if not lastUpdateTime[name] then
		timeSinceLastUpdate = 0.1
	else
		timeSinceLastUpdate = newTime - lastUpdateTime[name]
	end
	lastUpdateTime[name] = newTime
	seconds[name] = (seconds[name] or 0 ) + timeSinceLastUpdate

	local overheal = getOverheal( victimid, amount )
	healingDone[name] = ( healingDone[name] or 0 ) + amount	- overheal	
		
	local newHPS = math.floor(healingDone[name] / seconds[name])

	if name == ParserLib_SELF then
		broadcastdirty = newHPS - (data[playerName] or 0)
		data[playerName] = newHPS
		dirty = true
	elseif not Violation:UserViolation( name, moduleName ) then
		self:OnRemoteData( name, newHPS )			
	end
end

function mod:GetData(player)
	if player then
		return data[player]
	else
		return data
	end
end

function mod:CheckPartyCombat()
	for name, unit in pairs( roster.roster ) do
		if isInCombat[name] and not UnitAffectingCombat( unit.unitid ) then
			isInCombat[name] = nil
			lastUpdateTime[name] = nil
		elseif not isInCombat[name] and UnitAffectingCombat( unit.unitid ) then
			isInCombat[name] = true
		end
	end
end
