-----------------------------------------------------------------------
-- Locale
-----------------------------------------------------------------------
local L = AceLibrary("AceLocale-2.2"):new("Violation: HealingTaken")

L:RegisterTranslations("enUS", function() return {
	["Healing taken"] = true,
	["HT"] = true,
} end)

L:RegisterTranslations("koKR", function() return {
	["Healing taken"] = "받은 치유량",
} end)

-----------------------------------------------------------------------
-- Locals
-----------------------------------------------------------------------

local playerName = nil

local data = nil
local position = nil
local dirty = nil
local broadcastdirty = 0
local moduleName = "HealingTaken"

local parser = ParserLib:GetInstance("1.1")
local roster = AceLibrary("Roster-2.1")

-----------------------------------------------------------------------
-- Module declaration
-----------------------------------------------------------------------

local mod = Violation:NewModule(moduleName)
mod.revision = tonumber(("$Revision: 35576 $"):sub(12, -3)) or 1
mod.version = 4

-----------------------------------------------------------------------
-- Module initialization
-----------------------------------------------------------------------

function mod:OnModuleRegister()
	playerName = UnitName("player")
	data = self:new()
end

function mod:OnModuleEnable()
	parser:RegisterInfoType( "Violation "..moduleName, "heal", function( infoType, info ) self:OnParserEvent( infoType, info ) end )
	parser:RegisterInfoType( "Violation "..moduleName, "drain",function( infoType, info ) self:OnParserEvent( infoType, info ) end  )
	parser:RegisterInfoType( "Violation "..moduleName, "leech", function( infoType, info ) self:OnParserEvent( infoType, info ) end  )
end

function mod:OnModuleDisable()
	parser:UnregisterAllInfoTypes( "Violation "..moduleName )
end

function mod:OnReset()
	if type(data) == "table" then
		data = self:del(data)
	end

	data = self:new()
	data[playerName] = 0
	broadcastdirty = 0
	dirty = nil
end

-----------------------------------------------------------------------
-- Module implementation
-----------------------------------------------------------------------

function mod:GetRevision() return self.revision end
function mod:GetDisplayName() return L["Healing taken"] end
function mod:GetShortDisplayName() return L["HT"] end
function mod:GetBroadcastData()
	local x = broadcastdirty
	broadcastdirty = 0
	return x
end

function mod:IsBroadcastDirty()
	return broadcastdirty > 0 and true or false
end

function mod:IsDirty()
	return dirty
end

function mod:ResetDirty()
	dirty = nil
end

function mod:OnRemoteData(sender, input)
	data[sender] = (data[sender] or 0) + input
	dirty = true
end

local function getOverheal( unit, amount )
	local overheal = 0
	if unit and UnitExists(unit) then
		local d = UnitHealthMax(unit)-UnitHealth(unit)
		if d < amount then
			overheal = amount - d
		end
	end
	return overheal
end

function mod:OnParserEvent( infoType, info )
	local victim, amount
	if infoType == "heal" or ( ( infoType == "drain" or infoType == "leech" ) and info.attribute == "Health" ) then
		amount = info.amount
		victim = info.victim
	end
	if not victim or victim == ParserLib_NONE or not amount or amount == 0 then
		return
	end
	local overheal
	if victim == ParserLib_SELF then
		overheal = getOverheal( "player", amount )
		data[playerName] = data[playerName] + amount - overheal
		broadcastdirty = broadcastdirty + amount - overheal
		dirty = true
	else
		local unitid = roster:GetUnitIDFromName( victim )
		if not unitid then return end -- we're only interested in known friendlies
		if not Violation:UserViolation( victim, moduleName ) then
			overheal = getOverheal( unitid, amount )
			self:OnRemoteData( victim, amount - overheal )			
		end
	end
end

function mod:GetData(player)
	if player then
		return data[player]
	else
		return data
	end
end

